@@ -107,12 +107,6 @@ public class OutputBuffer extends Writer {
     private boolean doFlush = false;
 
 
-    /**
-     * Encoding to use.
-     */
-    private String enc;
-
-
     /**
      * Current char to byte converter.
      */
@@ -133,16 +127,6 @@ public class OutputBuffer extends Writer {
 
     // ----------------------------------------------------------- Constructors
 
-    /**
-     * Default constructor. Allocate the buffer with the default buffer size.
-     */
-    public OutputBuffer() {
-
-        this(DEFAULT_BUFFER_SIZE);
-
-    }
-
-
     /**
      * Create the buffer with the specified initial size.
      *
@@ -224,8 +208,6 @@ public class OutputBuffer extends Writer {
             conv.recycle();
             conv = null;
         }
-
-        enc = null;
     }
 
 
@@ -345,6 +327,9 @@ public class OutputBuffer extends Writer {
         if (closed) {
             return;
         }
+        if (coyoteResponse == null) {
+            return;
+        }
 
         // If we really have something to write
         if (buf.remaining() > 0) {
@@ -362,6 +347,7 @@ public class OutputBuffer extends Writer {
                 // An IOException on a write is almost always due to
                 // the remote client aborting the request. Wrap this
                 // so that it can be handled better by the error dispatcher.
+                coyoteResponse.setErrorException(e);
                 throw new ClientAbortException(e);
             }
         }
@@ -563,23 +549,16 @@ public class OutputBuffer extends Writer {
     }
 
 
-    /**
-     * @param s New encoding value
-     *
-     * @deprecated This method will be removed in Tomcat 9.0.x
-     */
-    @Deprecated
-    public void setEncoding(String s) {
-        enc = s;
-    }
-
-
     public void checkConverter() throws IOException {
         if (conv != null) {
             return;
         }
 
-        Charset charset = coyoteResponse.getCharset();
+        Charset charset = null;
+
+        if (coyoteResponse != null) {
+            charset = coyoteResponse.getCharset();
+        }
 
         if (charset == null) {
             if (coyoteResponse.getCharacterEncoding() != null) {
@@ -587,11 +566,7 @@ public class OutputBuffer extends Writer {
                 // Trigger an UnsupportedEncodingException
                 charset = B2CConverter.getCharset(coyoteResponse.getCharacterEncoding());
             }
-            if (enc == null) {
-                charset = org.apache.coyote.Constants.DEFAULT_BODY_CHARSET;
-            } else {
-                charset = getCharset(enc);
-            }
+            charset = org.apache.coyote.Constants.DEFAULT_BODY_CHARSET;
         }
 
         conv = encoders.get(charset);
@@ -603,38 +578,10 @@ public class OutputBuffer extends Writer {
     }
 
 
-    private static Charset getCharset(final String encoding) throws IOException {
-        if (Globals.IS_SECURITY_ENABLED) {
-            try {
-                return AccessController.doPrivileged(new PrivilegedExceptionAction<Charset>() {
-                    @Override
-                    public Charset run() throws IOException {
-                        return B2CConverter.getCharset(encoding);
-                    }
-                });
-            } catch (PrivilegedActionException ex) {
-                Exception e = ex.getException();
-                if (e instanceof IOException) {
-                    throw (IOException) e;
-                } else {
-                    throw new IOException(ex);
-                }
-            }
-        } else {
-            return B2CConverter.getCharset(encoding);
-        }
-    }
-
-
     private static C2BConverter createConverter(final Charset charset) throws IOException {
         if (Globals.IS_SECURITY_ENABLED) {
             try {
-                return AccessController.doPrivileged(new PrivilegedExceptionAction<C2BConverter>() {
-                    @Override
-                    public C2BConverter run() throws IOException {
-                        return new C2BConverter(charset);
-                    }
-                });
+                return AccessController.doPrivileged(new PrivilegedCreateConverter(charset));
             } catch (PrivilegedActionException ex) {
                 Exception e = ex.getException();
                 if (e instanceof IOException) {
@@ -687,7 +634,6 @@ public class OutputBuffer extends Writer {
                 conv.recycle();
             }
             conv = null;
-            enc = null;
         }
         initial = true;
     }
@@ -798,12 +744,6 @@ public class OutputBuffer extends Writer {
         }
     }
 
-
-    public void setErrorException(Exception e) {
-        coyoteResponse.setErrorException(e);
-    }
-
-
     private void appendByteArray(byte src[], int off, int len) throws IOException {
         if (len == 0) {
             return;
@@ -919,4 +859,19 @@ public class OutputBuffer extends Writer {
     private void toWriteMode(Buffer buffer) {
         buffer.mark().position(buffer.limit()).limit(buffer.capacity());
     }
+
+
+    private static class PrivilegedCreateConverter implements PrivilegedExceptionAction<C2BConverter> {
+
+        private final Charset charset;
+
+        PrivilegedCreateConverter(Charset charset) {
+            this.charset = charset;
+        }
+
+        @Override
+        public C2BConverter run() throws IOException {
+            return new C2BConverter(charset);
+        }
+    }
 }
