@@ -30,12 +30,19 @@ import java.net.URL;
 import java.net.URLClassLoader;
 import java.security.AccessControlException;
 import java.util.Random;
+import java.util.concurrent.ExecutionException;
+import java.util.concurrent.ScheduledExecutorService;
+import java.util.concurrent.ScheduledFuture;
+import java.util.concurrent.ScheduledThreadPoolExecutor;
+import java.util.concurrent.TimeUnit;
 
 import javax.management.InstanceNotFoundException;
 import javax.management.MBeanException;
+import javax.management.MBeanServer;
 import javax.management.ObjectName;
 
 import org.apache.catalina.Context;
+import org.apache.catalina.Lifecycle;
 import org.apache.catalina.LifecycleException;
 import org.apache.catalina.LifecycleState;
 import org.apache.catalina.Server;
@@ -50,7 +57,9 @@ import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
 import org.apache.tomcat.util.ExceptionUtils;
 import org.apache.tomcat.util.buf.StringCache;
+import org.apache.tomcat.util.modeler.Registry;
 import org.apache.tomcat.util.res.StringManager;
+import org.apache.tomcat.util.threads.TaskThreadFactory;
 
 
 /**
@@ -131,7 +140,7 @@ public final class StandardServer extends LifecycleMBeanBase implements Server {
     /**
      * The set of Services associated with this Server.
      */
-    private Service services[] = new Service[0];
+    private Service[] services = new Service[0];
     private final Object servicesLock = new Object();
 
 
@@ -168,6 +177,40 @@ public final class StandardServer extends LifecycleMBeanBase implements Server {
 
     private final Object namingToken = new Object();
 
+    /**
+     * The number of threads available to process utility tasks in this service.
+     */
+    private int utilityThreads = 2;
+
+    /**
+     * The utility threads daemon flag.
+     */
+    private boolean utilityThreadsAsDaemon = false;
+
+    /**
+     * Utility executor with scheduling capabilities.
+     */
+    private ScheduledThreadPoolExecutor utilityExecutor = null;
+    private final Object utilityExecutorLock = new Object();
+
+    /**
+     * Utility executor wrapper.
+     */
+    private ScheduledExecutorService utilityExecutorWrapper = null;
+
+
+    /**
+     * Controller for the periodic lifecycle event.
+     */
+    private ScheduledFuture<?> periodicLifecycleEventFuture = null;
+    private ScheduledFuture<?> monitorFuture;
+
+
+    /**
+     * The lifecycle event period in seconds.
+     */
+    private int periodicEventDelay = 10;
+
 
     // ------------------------------------------------------------- Properties
 
@@ -356,6 +399,101 @@ public final class StandardServer extends LifecycleMBeanBase implements Server {
         this.catalina = catalina;
     }
 
+
+    @Override
+    public int getUtilityThreads() {
+        return utilityThreads;
+    }
+
+
+    /**
+     * Handles the special values.
+     */
+    private static int getUtilityThreadsInternal(int utilityThreads) {
+        int result = utilityThreads;
+        if (result <= 0) {
+            result = Runtime.getRuntime().availableProcessors() + result;
+            if (result < 2) {
+                result = 2;
+            }
+        }
+        return result;
+    }
+
+
+    @Override
+    public void setUtilityThreads(int utilityThreads) {
+        // Use local copies to ensure thread safety
+        int oldUtilityThreads = this.utilityThreads;
+        if (getUtilityThreadsInternal(utilityThreads) < getUtilityThreadsInternal(oldUtilityThreads)) {
+            return;
+        }
+        this.utilityThreads = utilityThreads;
+        synchronized (utilityExecutorLock) {
+            if (oldUtilityThreads != utilityThreads && utilityExecutor != null) {
+                reconfigureUtilityExecutor(getUtilityThreadsInternal(utilityThreads));
+            }
+        }
+    }
+
+
+    /*
+     * Callers must be holding utilityExecutorLock
+     */
+    private void reconfigureUtilityExecutor(int threads) {
+        // The ScheduledThreadPoolExecutor doesn't use MaximumPoolSize, only CorePoolSize is available
+        if (utilityExecutor != null) {
+            utilityExecutor.setCorePoolSize(threads);
+        } else {
+            ScheduledThreadPoolExecutor scheduledThreadPoolExecutor = new ScheduledThreadPoolExecutor(threads,
+                    new TaskThreadFactory("Catalina-utility-", utilityThreadsAsDaemon, Thread.MIN_PRIORITY));
+            scheduledThreadPoolExecutor.setKeepAliveTime(10, TimeUnit.SECONDS);
+            scheduledThreadPoolExecutor.setRemoveOnCancelPolicy(true);
+            scheduledThreadPoolExecutor.setExecuteExistingDelayedTasksAfterShutdownPolicy(false);
+            utilityExecutor = scheduledThreadPoolExecutor;
+            utilityExecutorWrapper = new org.apache.tomcat.util.threads.ScheduledThreadPoolExecutor(utilityExecutor);
+        }
+    }
+
+
+    /**
+     * Get if the utility threads are daemon threads.
+     *
+     * @return the threads daemon flag
+     */
+    public boolean getUtilityThreadsAsDaemon() {
+        return utilityThreadsAsDaemon;
+    }
+
+
+    /**
+     * Set the utility threads daemon flag. The default value is true.
+     *
+     * @param utilityThreadsAsDaemon the new thread daemon flag
+     */
+    public void setUtilityThreadsAsDaemon(boolean utilityThreadsAsDaemon) {
+        this.utilityThreadsAsDaemon = utilityThreadsAsDaemon;
+    }
+
+
+    /**
+     * @return The period between two lifecycle events, in seconds
+     */
+    public int getPeriodicEventDelay() {
+        return periodicEventDelay;
+    }
+
+
+    /**
+     * Set the new period between two lifecycle events in seconds.
+     *
+     * @param periodicEventDelay The period in seconds, negative or zero will disable events
+     */
+    public void setPeriodicEventDelay(int periodicEventDelay) {
+        this.periodicEventDelay = periodicEventDelay;
+    }
+
+
     // --------------------------------------------------------- Server Methods
 
 
@@ -422,9 +560,10 @@ public final class StandardServer extends LifecycleMBeanBase implements Server {
             // undocumented yet - for embedding apps that are around, alive.
             return;
         }
+        Thread currentThread = Thread.currentThread();
         if (getPortWithOffset() == -1) {
             try {
-                awaitThread = Thread.currentThread();
+                awaitThread = currentThread;
                 while (!stopAwait) {
                     try {
                         Thread.sleep(10000);
@@ -448,7 +587,7 @@ public final class StandardServer extends LifecycleMBeanBase implements Server {
         }
 
         try {
-            awaitThread = Thread.currentThread();
+            awaitThread = currentThread;
 
             // Loop waiting for a connection and a valid command
             while (!stopAwait) {
@@ -567,7 +706,7 @@ public final class StandardServer extends LifecycleMBeanBase implements Server {
 
 
     /**
-     * @return the set of Services defined within this Server.
+     * @return The array of Services defined within this Server.
      */
     @Override
     public Service[] findServices() {
@@ -578,7 +717,7 @@ public final class StandardServer extends LifecycleMBeanBase implements Server {
      * @return the JMX service names.
      */
     public ObjectName[] getServiceNames() {
-        ObjectName onames[] = new ObjectName[services.length];
+        ObjectName[] onames = new ObjectName[services.length];
         for (int i = 0; i < services.length; i++) {
             onames[i] = ((StandardService) services[i]).getObjectName();
         }
@@ -611,7 +750,7 @@ public final class StandardServer extends LifecycleMBeanBase implements Server {
                 // Ignore
             }
             int k = 0;
-            Service results[] = new Service[services.length - 1];
+            Service[] results = new Service[services.length - 1];
             for (int i = 0; i < services.length; i++) {
                 if (i != j) {
                     results[k++] = services[i];
@@ -686,10 +825,7 @@ public final class StandardServer extends LifecycleMBeanBase implements Server {
      */
     @Override
     public String toString() {
-        StringBuilder sb = new StringBuilder("StandardServer[");
-        sb.append(getPort());
-        sb.append(']');
-        return sb.toString();
+        return "StandardServer[" + getPort() + ']';
     }
 
 
@@ -705,8 +841,9 @@ public final class StandardServer extends LifecycleMBeanBase implements Server {
         try {
             // Note: Hard-coded domain used since this object is per Server/JVM
             ObjectName sname = new ObjectName("Catalina:type=StoreConfig");
-            if (mserver.isRegistered(sname)) {
-                mserver.invoke(sname, "storeConfig", null, null);
+            MBeanServer server = Registry.getRegistry(null, null).getMBeanServer();
+            if (server.isRegistered(sname)) {
+                server.invoke(sname, "storeConfig", null, null);
             } else {
                 log.error(sm.getString("standardServer.storeConfig.notAvailable", sname));
             }
@@ -731,8 +868,9 @@ public final class StandardServer extends LifecycleMBeanBase implements Server {
         try {
             // Note: Hard-coded domain used since this object is per Server/JVM
             ObjectName sname = new ObjectName("Catalina:type=StoreConfig");
-            if (mserver.isRegistered(sname)) {
-                mserver.invoke(sname, "store", new Object[] { context }, new String[] { "java.lang.String" });
+            MBeanServer server = Registry.getRegistry(null, null).getMBeanServer();
+            if (server.isRegistered(sname)) {
+                server.invoke(sname, "store", new Object[] { context }, new String[] { "java.lang.String" });
             } else {
                 log.error(sm.getString("standardServer.storeConfig.notAvailable", sname));
             }
@@ -770,6 +908,12 @@ public final class StandardServer extends LifecycleMBeanBase implements Server {
         fireLifecycleEvent(CONFIGURE_START_EVENT, null);
         setState(LifecycleState.STARTING);
 
+        // Initialize utility executor
+        synchronized (utilityExecutorLock) {
+            reconfigureUtilityExecutor(getUtilityThreadsInternal(utilityThreads));
+            register(utilityExecutor, "type=UtilityExecutor");
+        }
+
         globalNamingResources.start();
 
         // Start our defined Services
@@ -778,6 +922,28 @@ public final class StandardServer extends LifecycleMBeanBase implements Server {
                 service.start();
             }
         }
+
+        if (periodicEventDelay > 0) {
+            monitorFuture = getUtilityExecutor().scheduleWithFixedDelay(this::startPeriodicLifecycleEvent, 0, 60,
+                    TimeUnit.SECONDS);
+        }
+    }
+
+
+    private void startPeriodicLifecycleEvent() {
+        if (periodicLifecycleEventFuture == null || periodicLifecycleEventFuture.isDone()) {
+            if (periodicLifecycleEventFuture != null && periodicLifecycleEventFuture.isDone()) {
+                // There was an error executing the scheduled task, get it and log it
+                try {
+                    periodicLifecycleEventFuture.get();
+                } catch (InterruptedException | ExecutionException e) {
+                    log.error(sm.getString("standardServer.periodicEventError"), e);
+                }
+            }
+            periodicLifecycleEventFuture =
+                    getUtilityExecutor().scheduleAtFixedRate(() -> fireLifecycleEvent(Lifecycle.PERIODIC_EVENT, null),
+                            periodicEventDelay, periodicEventDelay, TimeUnit.SECONDS);
+        }
     }
 
 
@@ -791,6 +957,16 @@ public final class StandardServer extends LifecycleMBeanBase implements Server {
     protected void stopInternal() throws LifecycleException {
 
         setState(LifecycleState.STOPPING);
+
+        if (monitorFuture != null) {
+            monitorFuture.cancel(true);
+            monitorFuture = null;
+        }
+        if (periodicLifecycleEventFuture != null) {
+            periodicLifecycleEventFuture.cancel(false);
+            periodicLifecycleEventFuture = null;
+        }
+
         fireLifecycleEvent(CONFIGURE_STOP_EVENT, null);
 
         // Stop our defined Services
@@ -798,6 +974,14 @@ public final class StandardServer extends LifecycleMBeanBase implements Server {
             service.stop();
         }
 
+        synchronized (utilityExecutorLock) {
+            if (utilityExecutor != null) {
+                utilityExecutor.shutdownNow();
+                unregister("type=UtilityExecutor");
+                utilityExecutor = null;
+            }
+        }
+
         globalNamingResources.stop();
 
         stopAwait();
@@ -930,4 +1114,9 @@ public final class StandardServer extends LifecycleMBeanBase implements Server {
     protected String getObjectNameKeyProperties() {
         return "type=Server";
     }
+
+    @Override
+    public ScheduledExecutorService getUtilityExecutor() {
+        return utilityExecutorWrapper;
+    }
 }
